-------------------------------------------------------------
-- Name: MTSLUI_Tools									   --
-- Description: contains all shared functions regarding UI --
-------------------------------------------------------------
MTSLUI_TOOLS = {
	---------------------------------------------------------------------------------------
	-- Create a generic MTSLUI_FRAME
	--
	-- @type:			string		Type of the frame ("Frame", "Button", "Slider")
	-- @name:			string		The name of the frame
	-- @parent:			ojbect		The parentframe (can be nil)
	-- @template:		string		The name of the template to follow (can be nil)
	-- @width:			number		The width of the frame
	-- @height:			number		The height of the frame
	-- @has_backdrop	boolean		Frame has backgroound or not (can be nil)
	--
	-- returns			Frame		Returns the created frame
	----------------------------------------------------------------------------------------
	CreateBaseFrame = function (self, type, name, parent, template, width, height, has_backdrop)
		local generic_frame = CreateFrame(type, name, parent, template)
		generic_frame:SetWidth(width)
		generic_frame:SetHeight(height)
		generic_frame:SetParent(parent)
		-- Add a background to the frame if we want it
		if has_backdrop ~= nil and has_backdrop == true then
			generic_frame:SetBackdrop({
				bgFile = "Interface/Tooltips/UI-Tooltip-Background",
				edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
				tile = true,
				tileSize = 16,
				edgeSize = 16,
				insets = { left = 4, right = 4, top = 4, bottom = 4 }
			})
			--  Black background
			generic_frame:SetBackdropColor(0,0,0,1)
		end
		-- make sure mouse is captured on our window (NO clicking through)
		generic_frame:EnableMouse(1)
		-- Disable zooming in/out
		generic_frame:EnableMouseWheel(true)
		generic_frame:Show()
		-- return the frame
		return generic_frame
	end,

	----------------------------------------------------------------------------------------
	-- Creates a label for the given frame
	--
	-- @owner		Frame		The frame for which to create the label
	-- @text		String		The text to show on the label
	-- @left		Number		The left position where the label starts
	-- @top			Number		The top position where the label starts
	--
	-- returns		Object		The created label
	----------------------------------------------------------------------------------------
	CreateLabel = function (self, owner, text, left, top, font_size, position)
		local new_label = owner:CreateFontString(nil,"ARTWORK")
		new_label:SetFont(MTSLUI_FONTS.NAME, font_size)
		new_label:SetPoint(position, left, top)
		if text ~= nil or text ~= "" then
			new_label:SetText(MTSLUI_FONTS.COLORS.TEXT.TITLE .. text)
		end
		return new_label
	end,

	-----------------------------------------------------------------------------------------
	-- Create a generic ScrollFrame
	--
	-- @parent_class		ojbect		The parentclass
	-- @parent_frame		ojbect		The parentframe
	-- @width				number		The width of the frame
	-- @height				number		The height of the frame
	-- @has_backdrop		boolean		Frame has backgroound or not (can be nil)
	-- @slider_steps		number		The amount of steps the slider has
	-- @height_slider_step	number		The height of 1 step in the slider
	--
	-- returns				Frame		Returns the created frame
	----------------------------------------------------------------------------------------
	CreateScrollFrame = function (self, parent_class, parent_frame, width, height, has_backdrop, height_slider_step)
		local scroll_frame = MTSLUI_TOOLS:CreateBaseFrame("Frame", "", parent_frame, nil, width, height, has_backdrop)
		-- add the vertical slider on the right to the frame
		scroll_frame.slider = MTSL_TOOLS:CopyObject(MTSLUI_VERTICAL_SLIDER)
		scroll_frame.slider:Initialise(parent_class, scroll_frame, height, height_slider_step)
		-- Make the frame scrollable
		scroll_frame:EnableMouseWheel(true)
		-- add mousewheel event to scrollframe
		scroll_frame:SetScript("OnMouseWheel", function(event_frame, delta)
			-- Only scroll if delta is + or -
			if delta ~= nil then
				-- scroll up on positive delta
				if delta > 0 then
					event_frame.slider:ScrollUp()
				else
					event_frame.slider:ScrollDown()
				end
			end
		end)

		-- return the frame
		return scroll_frame
	end,

	----------------------------------------------------------------------------------------
	-- Prints info about addon to chat
	----------------------------------------------------------------------------------------
	PrintAboutMessage = function (self)
		print(MTSLUI_FONTS.COLORS.TEXT.TITLE .. MTSL_ADDON.NAME)
		print(MTSLUI_FONTS.COLORS.TEXT.TITLE .. MTSLUI_FONTS.TAB .. "Author: " .. MTSLUI_FONTS.COLORS.TEXT.NORMAL .. MTSL_ADDON.AUTHOR)
		print(MTSLUI_FONTS.COLORS.TEXT.TITLE .. MTSLUI_FONTS.TAB .. "Version: " .. MTSLUI_FONTS.COLORS.TEXT.NORMAL .. MTSL_ADDON.VERSION)
	end,

	----------------------------------------------------------------------------------------
	-- Prints help about addon to chat
	----------------------------------------------------------------------------------------
	PrintHelpMessage = function (self)
		self:PrintAboutMessage()
        print("/mstl" .. MTSLUI_FONTS.TAB .. MTSLUI_FONTS.TAB .. "Toggles the account wide frame - COMING SOON")
        print("/mstl account")
		print("/mstl db" .. MTSLUI_FONTS.TAB .. "Shows the database explorer window")
        print("/mstl database")
		print("/mstl about" .. MTSLUI_FONTS.TAB .. "Print information about this addon")
		print("/mstl help" .. MTSLUI_FONTS.TAB .. "Print how to use this addon")
	end,

	----------------------------------------------------------------------------------------
	-- Print a summary of data present in the addon
	----------------------------------------------------------------------------------------
	PrintSummary = function(self)
		print("Missing TradeSkills List - Summary addon data")
		print("Professions")
		for k, v in pairs(MTSL_DATA.AMOUNT_SKILLS) do
			print(MTSLUI_FONTS.COLORS.TEXT.TITLE .. k .. ": skills (" .. v .. "),  levels (" .. MTSL_DATA.TRADESKILL_LEVELS .. "), items (" .. MTSL_DATA.AMOUNT_ITEMS[k] .. ")")
		end
		print("Objects")
		for k, v in pairs(MTSL_DATA.AMOUNT_OBJECTS) do
			print(MTSLUI_FONTS.COLORS.TEXT.TITLE .. k .. ": " .. v)
		end
		print("Players")
		-- print realms
		for k in pairs(MTSL_PLAYERS) do
			print(MTSLUI_FONTS.COLORS.TEXT.TITLE .. k)
			for key in pairs(MTSL_PLAYERS[k]) do
				local char = MTSL_PLAYERS[k][key]
				local char_text = MTSLUI_FONTS.COLORS.TEXT.NORMAL .. MTSLUI_FONTS.TAB .. char.NAME .. " (" .. char.XP_LEVEL .. " " .. char.FACTION .. "): "
				if char.TRADESKILLS.PRIMARY == 0 then
					char_text = char_text .. " no primary profession learned, "
				else
					char_text = char_text .. char.TRADESKILLS.PRIMARY.NAME .."(level " .. char.TRADESKILLS.PRIMARY.SKILL_LEVEL .. ", learned skills: " .. ", missing skills: " .. MTSL_TOOLS:CountItemsInArray(char.TRADESKILLS.PRIMARY.MISSING_SKILLS) .. "),"
				end
				if char.TRADESKILLS.SECONDARY == 0 then
					char_text = char_text .. " no secundary profession learned, "
				else
					char_text = char_text .. char.TRADESKILLS.SECONDARY.NAME .. "(level " .. char.TRADESKILLS.SECONDARY.SKILL_LEVEL .. ", learned skills: " .. ", missing skills: " .. MTSL_TOOLS:CountItemsInArray(char.TRADESKILLS.SECONDARY.MISSING_SKILLS) .. "),"
				end
				if char.TRADESKILLS.COOKING == 0 then
					char_text = char_text .. " cooking not learned, "
				else
					char_text = char_text .. "cooking (level " .. char.TRADESKILLS.COOKING.SKILL_LEVEL .. ", learned skills: " .. ", missing skills: " .. MTSL_TOOLS:CountItemsInArray(char.TRADESKILLS.COOKING.MISSING_SKILLS) .. "),"
				end
				if char.TRADESKILLS.FIRST_AID == 0 then
					char_text = char_text .. " first aid not learned"
				else
					char_text = char_text .. " first aid (level " .. char.TRADESKILLS.FIRST_AID.SKILL_LEVEL .. ", learned skills: " .. ", missing skills: " .. MTSL_TOOLS:CountItemsInArray(char.TRADESKILLS.FIRST_AID.MISSING_SKILLS) .. "),"
				end
				print(char_text)
			end
		end
	end,

	------------------------------------------------------------------------------------------------
	-- Sets the locale used ingame
	--
	-- returns				Boolean		Flag indicating if our language is supported
	------------------------------------------------------------------------------------------------
	SetAddonLocale = function(self)
		local locale = GetLocale()
		if locale ~= "enUS" and locale ~= "enGB" then
			print(MTSLUI_FONTS.COLORS.TEXT.ERROR .. "MTSL does not support your locale " .. GetLocale() .. "! Only English (enUS or enGB) clients are supported!")
			return false
		end
		return true
	end
}